/*!
 *     @header BitmapView
 *
 *   @abstract A view that displays a static bitmap image.
 * @discussion
 *             Copyright &copy; 2000 David P. Kleinschmidt
 *
 *             This software is provided 'as-is', without any express or implied
 *             warranty. In no event will the author be held liable for any
 *             damages arising from the use of this software. Permission is
 *             granted to anyone to use this software for any purpose, including
 *             commercial applications, and to alter it and redistribute it
 *             freely, subject to the following restrictions:
 *
 *             1. The origin of this software must not be misrepresented; you
 *             must not claim that you wrote the original software. If you use
 *             this software in a product, an acknowledgment in the product
 *             documentation would be appreciated but is not required.
 *
 *             2. Altered source versions must be plainly marked as such, and
 *             must not be misrepresented as being the original software.
 *
 *             3. This notice may not be removed or altered from any source
 *             distribution.
 */

#ifndef BU_BITMAP_VIEW_H
#define BU_BITMAP_VIEW_H

#include <Bitmap.h>
#include <FlotsamDefs.h>
#include <Rect.h>
#include <SupportDefs.h>
#include <View.h>

namespace BeUnited
{

/*!
 *       @class BitmapView
 *
 *   @abstract A view that displays a static bitmap image.
 * @discussion This class is used to display a static bitmap image that does not
 *             respond to user input.  The bitmap can be either a BBitmap* or a
 *             bitmap in the application's resources, referenced by name.
 */
class BU_FLOTSAM_API BitmapView : public BView
{
public:

	/*!
	 *   @function BitmapView
	 *
	 *   @abstract Constructor
	 * @discussion Sets up a new BitmapView, using a given BBitmap*.
	 *      @param frame        See BView::BView().
	 *      @param name         See BView::BView().
	 *      @param bitmap       A pointer to the bitmap to display.  The
	 *                          BitmapView will assume ownership of this bitmap.
	 *                          If bitmap is NULL, an empty 32x32
	 *                          black-and-white bitmap will be created and used.
	 *      @param resizingMode See BView::BView().
	 *      @param flags        see BView::BView().  These flags will be |ed with
	 *                          B_WILL_DRAW.
	 */
	BitmapView( BRect frame, const char* name, BBitmap* bitmap,
	  uint32 resizingMode, uint32 flags );

	/*!
	 *   @function BitmapView
	 *
	 *   @abstract Constructor
	 * @discussion Sets up a new BitmapView, using the bitmap with a given name.
	 *      @param frame        See BView::BView().
	 *      @param name         See BView::BView().
	 *      @param bitmapName   Either a filename of a bitmap to display, or the
	 *                          name of a B_TRANSLATOR_BITMAP application
	 *                          resource to display.
	 *      @param resizingMode See BView::BView().
	 *      @param flags        see BView::BView().  These flags will be |ed
	 *                          with B_WILL_DRAW.
	 */
	BitmapView( BRect frame, const char* bitmapName, uint32 resizingMode,
	  uint32 flags );

	/*!
	 *   @function ~BitmapView
	 *
	 *   @abstract Destructor
	 * @discussion Deletes the BitmapView, freeing all memory associated with it
	 *             (including its bitmap).
	 */
	~BitmapView();

	/*!
	 *   @function Draw
	 *
	 *   @abstract Redraws the BitmapView.
	 * @discussion This function is called whenever the view needs to be redrawn.
	 *      @param updateRect The rectangle that needs updating, expressed in
	 *                        the view coordinate system.
	 */
	virtual void Draw( BRect updateRect );

	/*!
	 *   @function GetPreferredSize
	 *
	 *   @abstract Determines the view's preferred size.
	 *
	 * @discussion This function returns this BitmapView's preferred size
	 *             (namely, the size of the bitmap).
	 *      @param width  Pointer to a variable that will receive this view's
	 *                    preferred width.
	 *      @param height Pointer to a variable that will receive this view's
	 *                    preferred height.
	 */
	virtual void GetPreferredSize( float* width,
	  float* height );

	/*!
	 *   @function SetBitmap
	 *
	 *   @abstract Sets this view's bitmap.
	 * @discussion Sets the displayed bitmap to bitmap, and assumes ownership of
	 *             it. Deletes the view's previous bitmap.
	 *      @param bitmap The new bitmap to display.  This parameter may not be
	 *                    NULL.
	 */
	void SetBitmap( BBitmap* bitmap );

	/*!
	 *   @function GetBitmap
	 *
	 *   @abstract Returns the view's current bitmap.
	 * @discussion Returns a pointer to the view's current bitmap.  This is the
	 *             actual bitmap contained in the view; you may modify it but
	 *             you must not delete it.  If you modify it, you should call
	 *             Invalidate() afterwards.
	 *     @result The view's current bitmap.
	 */
	inline BBitmap* GetBitmap()
		{ return fBitmap; };

private:

	BBitmap* fBitmap;
};

}

#endif
