/*!
 *     @header HttpStream
 *
 *   @abstract A BDataIO that retrieves its data from the Web.
 * @discussion
 *             Copyright &copy; 2000 David P. Kleinschmidt
 *
 *             This software is provided 'as-is', without any express or implied
 *             warranty. In no event will the author be held liable for any
 *             damages arising from the use of this software. Permission is
 *             granted to anyone to use this software for any purpose, including
 *             commercial applications, and to alter it and redistribute it
 *             freely, subject to the following restrictions:
 *
 *             1. The origin of this software must not be misrepresented; you
 *             must not claim that you wrote the original software. If you use
 *             this software in a product, an acknowledgment in the product
 *             documentation would be appreciated but is not required.
 *
 *             2. Altered source versions must be plainly marked as such, and
 *             must not be misrepresented as being the original software.
 *
 *             3. This notice may not be removed or altered from any source
 *             distribution.
 */

#ifndef BU_HTTP_STREAM_H
#define BU_HTTP_STREAM_H

#define BU_BUILDING_FLOTSAM

#include <DataIO.h>
#include <FlotsamDefs.h>
#include <map>
#include <NetEndpoint.h>
#include <OS.h>
#include <string>
#include <SupportDefs.h>

namespace BeUnited
{

/*!
 *      @class HttpStream
 *
 *   @abstract A BDataIO that retrieves its data from the Web.
 * @discussion This class attempts to make it relatively easy to retrieve data
 *             from the World Wide Web.  It is based largely on
 *             <a href="http://www.python.org">Python's</a> httplib.  Here is an
 *             example of its usage:
 *             <pre>
 *             HttpStream h( "www.beunited.org" );
 *             char buffer[ 512 ];
 *             h.PutRequest( "GET", "/index.html" );
 *             h.PutHeader( "Host", "www.beunited.org" );
 *             h.PutHeader( "Accept", "text/html" );
 *             h.PutHeader( "Accept", "text/plain" );
 *             h.EndHeaders();
 *             h.GetReply();
 *             cout &lt;&lt; h.GetCode() &lt;&lt; endl;  // should be 200
 *             h.Read( buffer, 512 );
 *             </pre>
 */
class BU_FLOTSAM_API HttpStream : public BDataIO
{
public:
	// constants & type definitions

	/*!
	 *    @typedef HeaderMap
	 *
	 *   @abstract Maps header names to their values.
	 * @discussion A data member of this type is used to hold the headers
	 *             returned by the server.
	 */
	typedef map< string, string > HeaderMap;

	/*!
	 *        @var kBufferSize
	 *
	 *   @abstract The size of the HttpStream's internal buffer.
	 * @discussion The value of this constant is B_PAGE_SIZE.  If you want to
	 *             tweak this, the value must be a multiple of B_PAGE_SIZE
	 *             (4096) because areas are used to hold the incoming data.
	 */
	static const size_t kBufferSize;

	/*!
	 *        @var kHttpPort
	 *
	 *   @abstract The port HTTP servers run on.
	 * @discussion This is the default port that HTTP servers accept connections
	 *             on (port 80).
	 */
	static const int kHttpPort;

	/*!
	 *        @var kHttpVersion
	 *
	 *   @abstract The version of HTTP HttpStream implements.
	 * @discussion The value of this constant is "HTTP/1.0", as HttpStream only
	 *             implements the functionality of a version 1.0 client.
	 */
	static const char* const kHttpVersion;

public:
	// construction & destruction

	/*!
	 *   @function HttpStream
	 *
	 *   @abstract Default constructor.
	 * @discussion Creates an HttpStream object, optionally connecting to the
	 *             given host on the given port.
	 *      @param host The host to connect to.  If the host is not given, no
	 *                  connection is established until Connect() is called.
	 *      @param port The port to connect to.  If port is 0, host is searched
	 *                  to see if it contains a port, and if it doesn't,
	 *                  kHttpPort is used.
	 */
	HttpStream( const string& host = B_EMPTY_STRING,
	  int port = 0 );

	/*!
	 *   @function ~HttpStream
	 *
	 *   @abstract Destructor.
	 * @discussion Handles the destruction and cleanup of an HttpStream, closing
	 *             the connection, freeing memory, etc.
	 */
	~HttpStream();

public:
	// I/O Functions

	/*!
	 *   @function Read
	 *
	 *   @abstract Reads from the HTTP connection into a buffer.
	 * @discussion This function will return the next bytes available from the
	 *             server, up to and including numBytes bytes.  If fewer bytes
	 *             are available, it will not wait for more to fill the buffer.
	 *             If no data is available, this function blocks until some
	 *             data has arrived, at which point that data will be delivered.
	 *      @param buffer   The buffer to read into.
	 *      @param numBytes The maximum number of bytes to read into the buffer.
	 *     @result The number of bytes read; 0 if the connection has been closed
	 *             and no more data remains to be read.
	 */
	virtual ssize_t Read( void* buffer, size_t numBytes );

	/*!
	 *   @function Write
	 *
	 *   @abstract Sends an arbitrary string to the remote host.
	 * @discussion This function will send the contents of the given buffer to
	 *             the remote server.  Note that if you want to send data to the
	 *             server, this must be called after EndHeaders() but before
	 *             GetReply().
	 *      @param buffer   The buffer to write from.
	 *      @param numBytes The number of bytes to write.
	 *     @result The number of bytes written, which is equal to numBytes if
	 *             the connection is open and the buffer was sent successfully.
	 *             The result will be 0 if the connection has been closed.
	 */
	virtual ssize_t Write( const void* buffer, size_t numBytes );

	/*!
	 *   @function ReadLine
	 *
	 *   @abstract Reads the next line.
	 * @discussion This function retrieves the next line of text from the remote
	 *             host (as determined by the receipt of a line feed character),
	 *             up to but not including the trailing line feed.
	 *      @param removeCr If the line ends in a carriage return-line feed
	 *                      sequence (CRLF), the CR will be removed if removeCr
	 *                      is true.  if removeCr is false, only the LF will be
	 *                      removed.  This is useful if you are expecting binary
	 *                      data.
	 *     @result Returns the next line of data.
	 */
	string ReadLine( bool removeCr = true );

	/*!
	 *   @function ReadLine
	 *
	 *   @abstract Reads the next line.
	 * @discussion This function retrieves the next line of text from the remote
	 *             host (as determined by the receipt of a line feed character),
	 *             up to but not including the trailing line feed.
	 *      @param buffer   The string to read the next line into.
	 *      @param removeCr If the line ends in a carriage return-line feed
	 *                      sequence (CRLF), the CR will be removed if removeCr
	 *                      is true.  if removeCr is false, only the LF will be
	 *                      removed.  This is useful if you are expecting binary
	 *                      data.
	 */
	void ReadLine( string& buffer, bool removeCr = true );

	/*!
	 *   @function ReadUntil
	 *
	 *   @abstract Reads until a certain line is read.
	 * @discussion This function reads into the given buffer until a line
	 *             that exactly matches the supplied line is read.  If the line
	 *             is never encountered, the rest of the data in the buffer is
	 *             fetched.  The matched line is removed from the stream.
	 *      @param buffer    The buffer to read into.
	 *      @param line      The line to read until.
	 *      @param removeCrs If true, lines in the buffer are guaranteed to end
	 *                       with only a line feed.  If false, they may end with
	 *                       a carriage return-line feed sequence.
	 */
	void ReadUntil( string& buffer, const string& line,
	  bool removeCrs = true );

public:
	// HTTP functions

	/*!
	 *   @function Close
	 *
	 *   @abstract Closes the HTTP connection.
	 * @discussion If the connection is open, this function closes it; otherwise
	 *             it does nothing.  Connections are automatically closed when
	 *             HttpStream objects are destroyed.
	 */
	void Close();

	/*!
	 *   @function Connect
	 *
	 *   @abstract Establishes a connection with a remote host.
	 * @discussion This function connects to a web server with the given
	 *             hostname at the given port number.  If the connection fails,
	 *             GetCode() will return -1 and GetMessage will return a
	 *             description of what went wrong.
	 *      @param host The host to connect to.  For convenience, this may be
	 *                  a domain name ("www.beunited.org") or an IP address in
	 *                  dotted-quad notation ("127.0.0.1").  It may also contain
	 *                  the port number as is specified in URLs
	 *                  ("www.beunited.org:80"), as long as the parameter port
	 *                  is 0.
	 *      @param port The port to connect to.  If port == 0, host is searched
	 *                  to see if it contains a port number as explained above.
	 *                  If not, kHttpPort is assumed.
	 */
	void Connect( const string& host, int port = 0 );

	/*!
	 *   @function EndHeaders
	 *
	 *   @abstract Signals the end of the outgoing HTTP headers.
	 * @discussion You must call this function exactly once: after you've made
	 *             your request and sent all of your headers.  You will not get
	 *             a reply until this call has been made.  If you call
	 *             EndHeaders before you're done sending all the headers, those
	 *             that are sent afterwards will likely be ignored.
	 */
	inline void EndHeaders()
		{ Write( "\r\n", 2 ); };

	/*!
	 *   @function GetReply
	 *
	 *   @abstract Begins receiving the HTTP response from the remote host.
	 * @discussion You must call this function exactly once: after you've called
	 *             EndHeaders() and, optionally, Write(), and before you call
	 *             GetCode(), GetMessage(), GetHeader(), or any of the Read...()
	 *             functions.
	 */
	void GetReply();

	/*!
	 *   @function PutHeader
	 *
	 *   @abstract Sends an HTTP header to the server.
	 * @discussion If you call this function, you must do so after you call
	 *             PutRequest(), but before you call EndHeaders().
	 *      @param header The header description (ie, "User-Agent").
	 *      @param arg    The header's value.
	 */
	void PutHeader( const string& header, const string& arg );

	/*!
	 *   @function PutRequest
	 *
	 *   @abstract Makes an HTTP request of the server.
	 * @discussion This function must be called exactly once: After the
	 *             connection has been established, but before PutHeader() is
	 *             called.
	 *      @param request  What kind of request to make.  This parameter is
	 *                      usually "GET" to receive web pages, or occasionally
	 *                      "PUT" to send form data.
	 *      @param selector The requested document's path.  If there is a query
	 *                      string, it gets put here (ie,
	 *                      "/cgi-bin/query.cgi?name=Dave%20Kleinschmidt").
	 */
	void PutRequest( const string& request, const string& selector = "/" );

public:
	// accessors

	/*!
	 *   @function GetHeader
	 *
	 *   @abstract Returns the value of a header sent back from the server.
	 *     @result The value of the header, or, if the server did not send a
	 *             header with that name, an empty string.
	 */
	inline string GetHeader( const string& header )
		{ return fHeaders[ ToLower( header ) ]; };

	/*!
	 *   @function Eof
	 *
	 *   @abstract Returns whether you've reached the end of the stream.
	 * @discussion You have reached the end of the stream if the connection has
	 *             been closed and there is no more data left to be read.
	 *     @result True if all the data has been read from the stream, or false
	 *             if there may be more pending.  Note that if the server has
	 *             finished sending data but the connection has not been closed,
	 *             Eof() will return false.
	 */
	inline bool Eof()
		{ return fEof; };

	/*!
	 *   @function GetCode
	 *
	 *   @abstract Returns the HTTP result code.
	 * @discussion This function returns the status code sent back from the
	 *             server after our request was made.  For a complete listing
	 *             of codes, look up <a
	 *             href=http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html>
	 *             RFC 2616, section 10</a> for a complete listing of possible
	 *             result codes.  The most common are 200 (OK) and 404 (Not
	 *             Found).
	 *     @result The HTTP result code sent by the server, or -1 if either a
	 *             client-side error occurred (like specifying a nonexistant
	 *             server or a bad port number) or the result has not yet been
	 *             returned.
	 */
	inline int GetCode()
		{ return fCode; };

	/*!
	 *   @function GetMessage
	 *
	 *   @abstract Returns the HTTP reply message.
	 * @discussion This is a message that the server sends back that provides a
	 *             short description of the status code.  For a complete
	 *             listing look up <a
	 *             href=http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html>
	 *             RFC 2616, section 10</a>.  The most common are "OK" and
	 *             "Not Found".  Note that the strings in the RFC are only
	 *             suggestions that should be used for displaying messages to
	 *             the user; any decisions based on status should use the result
	 *             code instead.
	 *     @result The message sent along with the HTTP result code, or an empty
	 *             string if a client-side error has occurred (bad server name
	 *             or port, for example) or the result has not yet been fetched.
	 */
	inline string GetMessage()
		{ return fMessage; };

	/*!
	 *   @function SetDebugLevel
	 *
	 *   @abstract Turns debugging on or off.
	 * @discussion If debugging is on, all sorts of miscellaneous status
	 *             information is printed to stdout.
	 *      @param debugLevel True to turn debugging on, false to turn it off.
	 */
	inline void SetDebugLevel( bool debugLevel )
		{ fDebugLevel = debugLevel; };

private:
	// helpers

	inline static bool IsWhitespace( char myChar )
		{ return string( " \t\n\r\f\v" ).find( myChar ) != 0; };
	static string ToLower( const string& other );

private:
	// pay no attention to the man behind the curtain...
	char*        fBuffer;
	area_id      fBufferAreaID;
	char*        fBufferCursor;
	char*        fBufferEnd;
	int          fCode;
	bool         fDebugLevel;
	bool         fEof;
	HeaderMap    fHeaders;
	string       fMessage;
	BNetEndpoint fServer;
	string       fVersion;
};

}

#endif // ndef BU_HTTP_STREAM_H
