/*!
 *     @header HttpCat
 *
 *   @abstract Sample code demonstrating Flotsam's HttpStream class.
 * @discussion
 *             This program demonstrates how to use Flotsam's HttpStream class
 *             to download a web page.  You invoke it on the command line like
 *             so:
 *
 *             <pre>&gt; HttpCat url-of-web-page</pre>
 *
 *             HttpCat will then download the web page and display it, line by
 *             line.
 *
 *             <hr>
 *
 *             Copyright &copy; 2000 David P. Kleinschmidt
 *
 *             This software is provided 'as-is', without any express or implied
 *             warranty. In no event will the author be held liable for any
 *             damages arising from the use of this software. Permission is
 *             granted to anyone to use this software for any purpose, including
 *             commercial applications, and to alter it and redistribute it
 *             freely, subject to the following restrictions:
 *
 *             1. The origin of this software must not be misrepresented; you
 *             must not claim that you wrote the original software. If you use
 *             this software in a product, an acknowledgment in the product
 *             documentation would be appreciated but is not required.
 *
 *             2. Altered source versions must be plainly marked as such, and
 *             must not be misrepresented as being the original software.
 *
 *             3. This notice may not be removed or altered from any source
 *             distribution.
 */

#include <HttpStream.h>
#include <iostream>

/*!
 *       @enum Error
 *
 *   @abstract The different errors that can occur.
 *   @constant BAD_COMMAND_LINE The command line specified is of the incorrect
 *                              form.
 *   @constant CLIENT_ERROR     A client-side error occurred (ie, server not
 *                              found, nonnumeric port, etc.).
 *   @constant SERVER_ERROR     A server-side error occurred (ie, file not
 *                              found, permission denied, etc.).
 *   @constant URL_NOT_HTTP     The supplied URL did not specify the HTTP
 *                              protocol.
 *   @constant SNAFU            "Situation normal..."
 */
enum Error
{
	BAD_COMMAND_LINE = 'CmdL',
	CLIENT_ERROR     = 'Clnt',
	SERVER_ERROR     = 'Srvr',
	URL_NOT_HTTP     = 'Http',
	SNAFU            = 'Oops'
};

/*!
 *   @function main
 *
 *   @abstract HttpCat's entry point.
 * @discussion See the file <a href="../../../HttpCat.cpp">HttpCat.cpp</a> for
 *             a play-by-play of how to use HttpStream.
 *      @param argc The number of command line arguments.  This must equal 2.
 *      @param argv The command line arguments.<br>
 *                  argv[ 0 ] must be the name of the executable.<br>
 *                  argv[ 1 ] must be an HTTP URL specifying an HTML document.
 *     @result EXIT_SUCCESS if all goes well, or EXIT_FAILURE if an error
 *             occurred.
 */
int main( int argc, char* argv[] );

//
// main
//
int main( int argc, char* argv[] )
{
	string               url;
	string               host;
	string               selector;
	int                  i;
	BeUnited::HttpStream connection;

	try
	{
		if( argc != 2 )
			throw BAD_COMMAND_LINE;

		// parse URL
		url = argv[ 1 ];
		host = url;
		if( url.substr( 0, 7 ) != "http://" )
			throw URL_NOT_HTTP;

		host = host.substr( 7, host.size() - 7 );
		i = host.find( '/' );
		if( i > 0 )
		{
			selector = host.substr( i, host.size() - i );
			host = host.substr( 0, i );
		}

		// This call establishes a connection to the given host.  Due to our
		// url parsing scheme, host may contain a port number (ie, it may be
		// "www.beunited.org:80").  This is OK, as Connect will worry about
		// this for us.
		connection.Connect( host );

		// Connect will set fCode to -1 if it fails, and fMessage will reflect
		// the reason.
		if( connection.GetCode() == -1 )
			throw CLIENT_ERROR;

		// Send our request.  This must be the first thing you do with the
		// connection.  "GET" is the method most commonly used for retrieving
		// web pages.
		connection.PutRequest( "GET", selector );

		// Send some headers.
		// The first one tells the host which system we're trying to access,
		// so that if it's running multiple web servers (virtual hosts) it'll
		// know which we're trying to access.
		connection.PutHeader( "Host", host );

		// The second and third say we'd only like to receive html or plain
		// text documents.  These headers are, unfortunately, occasionally
		// ignored.
		connection.PutHeader( "Accept", "text/html" );
		connection.PutHeader( "Accept", "text/plain" );

		// The next identifies our client software to the server.
		connection.PutHeader( "User-Agent", "HtmlStripper/0.5" );

		// That's all.  Now we must call EndHeaders().
		connection.EndHeaders();

		// If we were doing any HTTP upload, we'd do it here, with Write().
		// But we're not.  So we now go on to receiving the reply.
		connection.GetReply();

		// All result codes are between 100 and 999.  We're only interested if
		// the connection's status is between 200 and 299 (Successful codes).
		// Technically, other status codes are not necessarily errors and
		// should not be treated as such by more robust clients.
		if( connection.GetCode() < 200 || connection.GetCode() >= 300 )
			throw SERVER_ERROR;

		while( !connection.Eof() )
		{
			cout << connection.ReadLine() << endl;
		}

		// note that Eof does not mean that the connection's been closed.  You
		// should close the connection after you're done.
		connection.Close();
	}
	catch( Error err )
	{
		switch( err )
		{
		case BAD_COMMAND_LINE:
			cerr << "Usage: " << argv[ 0 ] << " url-of-web-page\n";
			break;
		case CLIENT_ERROR:
			// If a client error occurs, the connection's message is set
			// appropriately.
			cerr << "Error: " << connection.GetMessage() << endl;
			break;
		case SERVER_ERROR:
			cerr << "Error #" << connection.GetCode() << ": " <<
			  connection.GetMessage() << endl;
			break;
		case URL_NOT_HTTP:
			cerr << "HttpCat only deals with HTTP URLs.\n";
			break;
		case SNAFU:
		default:
			cerr << "An unknown error has occurred.\n";
			break;
		}

		// It's safe to call Close if the connection isn't open.
		connection.Close();
		return EXIT_FAILURE;
	}
	return EXIT_SUCCESS;
}
